from django.db import models


# Create your models here.
class Application(models.Model):
    name = models.CharField(max_length=50)
    auth_url = models.URLField(max_length=500, help_text='URL that authenticated uploader')
    receiving_url = models.URLField(max_length=500, help_text='URL that receives the file')
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)

    def __str__(self):
        return self.name



# Generated by Django 3.0.6 on 2020-05-22 15:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bloxby', '0002_page_template_templateasset'),
    ]

    operations = [
        migrations.AddField(
            model_name='template',
            name='remote_id',
            field=models.IntegerField(blank=True, help_text='ID of the template on the remote server.', null=True),
        ),
        migrations.AlterField(
            model_name='page',
            name='name',
            field=models.CharField(blank=True, max_length=500, null=True),
        ),
    ]



from django.urls import path

from bloxby import views

urlpatterns = [
    path('ftp/receive/', views.ReceiveFTPItemsView.as_view(), name='receive'),
    path('ftp/auth/', views.AuthFTPUserView.as_view(), name='auth'),
    path('page/', views.PageRenderView.as_view(), name='render-page'),
    path('import-site/', views.ImportSiteView.as_view(), name='import-site'),
    path('', views.TestIndexView.as_view(), name='index-test')
]



from django.contrib.sites.models import Site
from pyftpdlib.servers import FTPServer

from bloxby.ftp.authorizers import APIAuth
from bloxby.ftp.filesystems import CustomFileSystem
from bloxby.ftp.handlers import Handler


def get_ftp_server(host, port=21):
    authorizer = APIAuth()
    handler = Handler
    handler.authorizer = authorizer
    handler.abstracted_fs = CustomFileSystem
    handler.masquerade_address = Site.objects.get_current().domain
    handler.passive_ports = range(60000, 65535)
    address = (host, port)
    server = FTPServer(address, handler)
    return server



import json

import requests
from .generics import Generic


class Package(Generic):
    path = '/api/packages/'

    def create(
        self, name, sites_number=20, hosting_option=None, price=0, subscription='Monthly', currency='USD',
        export_site=True, ftp_publish=True, disk_space=100, templates=None, blocks=None, status='Active'
    ):
        if not hosting_option:
            hosting_option = [
                'Sub-Folder'
            ]
        data = [
            ('name', f'{self.package_prefix}{name}'),
            ('sites_number',  sites_number),
            ('price', price),
            ('subscription', subscription),
            ('currency', currency),
            ('export_site', self.bool_cast(export_site)),
            ('ftp_publish', self.bool_cast(ftp_publish)),
            ('disk_space', disk_space),
            ('status', status)
        ]
        if templates:
            data.append(('templates', json.dumps(templates)))
        if blocks:
            data.append(('blocks', json.dumps(blocks)))
        if hosting_option:
            data.append(('hosting_option', json.dumps(hosting_option)))
        response = requests.post(
            f'{self.base_url}{self.path}', data=data, headers=self.get_headers(), auth=(self.get_auth())
        )
        return response.json(), response.status_code in [200, 201]

    def update(self, package_id, **kwargs):
        for key, value in kwargs.items():
            if isinstance(value, bool):
                kwargs[key] = self.bool_cast(value)
            if isinstance(value, list):
                kwargs[key] = json.dumps(value)
            if key == 'name' and self.package_prefix not in value:
                kwargs[key] = f'{self.package_prefix}{value}'
        response = requests.put(
            f'{self.base_url}{self.path}{package_id}/', data=kwargs, headers=self.get_headers(), auth=self.get_auth()
        )
        return response.json(), response.status_code in [201, 200]

    def delete(self, package_id):
        response = requests.delete(
            f'{self.base_url}{self.path}{package_id}/', headers=self.get_headers(), auth=self.get_auth()
        )
        return response.status_code == 200

    def retrieve(self, package_id):
        response = requests.get(
            f'{self.base_url}{self.path}{package_id}/', headers=self.get_headers(), auth=self.get_auth()
        )
        data = response.json()
        is_success = response.status_code == 200
        return data[0] if is_success else data, is_success

    def all(self):
        response = requests.get(
            f'{self.base_url}{self.path}/', headers=self.get_headers(), auth=self.get_auth()
        )
        data = response.json()
        is_success = response.status_code == 200
        return data if is_success else data, is_success



import requests
from pyftpdlib.authorizers import DummyAuthorizer, AuthenticationFailed

from bloxby.ftp.models import Application


class APIAuth(DummyAuthorizer):
    """
    Authentication is in the format:
    username: app_name|bloxby_auth_id|target|obj_id
    password: user_password
    # Target could be 'home, event'
    # Object id carries the id of the event, if it's a home page to an event, otherwise 'null'
    """
    def validate_authentication(self, username, password, handler):
        msg = 'Authentication Failed'
        app_and_username_and_target = username.lower().split('|')
        if len(app_and_username_and_target) != 4:
            raise AuthenticationFailed('Invalid username')
        app_name = app_and_username_and_target[0]
        username = app_and_username_and_target[1]
        target = app_and_username_and_target[2]
        obj_id = app_and_username_and_target[3]
        try:
            # Meta is going to be anything that will assist us in identifying the target
            application = Application.objects.get(name__iexact=app_name)
        except Application.DoesNotExist:
            raise AuthenticationFailed(msg)
        response = requests.post(application.auth_url, data={'username': username, 'password': password})
        if 199 < response.status_code < 300:
            setattr(handler, 'target', target)
            setattr(handler, 'user', username)
            setattr(handler, 'obj_id', obj_id)
            setattr(handler, 'application', application)
            setattr(handler, 'user_home', str(app_name) + str(username) + str(target))
            return None
        raise AuthenticationFailed(msg)

    def get_msg_login(self, username):
        return f'Welcome {username}'

    def has_user(self, username):
        return True

    def get_home_dir(self, username):
        return ''

    def has_perm(self, username, perm, path=None):
        return perm in self.get_perms(username)

    def get_perms(self, username):
        return 'mwaler'



from django.db import models

from .functions import Bloxby

bloxby = Bloxby()


class UserBridgeQuerySet(models.QuerySet):
    def create(self, **kwargs):
        if not kwargs.pop('override'):
            raise Exception('Use UserBridge.create(bloxby_package_id, django_user) instead.')
        return super(UserBridgeQuerySet, self).create(**kwargs)

    def delete(self):
        qs = self._chain()
        for item in qs:
            # This calls the model delete for each, might not be query efficient, but this is what we want
            item.delete()



import re
from zipfile import ZipFile

import bs4
from django.core.files.base import ContentFile

LINK_PATTERN = '[\.*?\/?\w+/\-_]{1,}\.[\w+]{2,}'

CSS_LINK_PATTERN = 'url\([./]*?[\'"]?[\w+/.]*[?\w+]*[\'"]?\)'


def replace_links(page):
    from bloxby.models import TemplateAsset, Page
    """Replace the links in the page here"""
    soup = bs4.BeautifulSoup(page.html.read(), 'html.parser')
    css_assets = []
    for link in soup.find_all('link'):
        if link.get('href'):
            # Stylesheets, fonts, favicon
            href = link['href']
            try:
                asset = TemplateAsset.objects.get(initial_path=href, template=page.template)
            except TemplateAsset.DoesNotExist:
                asset = None
            except TemplateAsset.MultipleObjectsReturned:
                asset = TemplateAsset.objects.filter(initial_path=href, template=page.template).first()
            if asset:
                link['href'] = asset.file.url
                if '.css' in href:
                    css_assets.append(asset)
    html_content = soup.prettify()
    # Find all other asset embeddings in the HTML
    links = re.findall(LINK_PATTERN, html_content)
    for link in links:
        search_path = link.replace('../', '')
        search_path = search_path.split('?')[0]
        search_path = search_path.split('#')[0]
        if search_path.endswith('.html'):
            try:
                p = Page.objects.get(template=page.template, name=search_path)
                html_content = html_content.replace(link, p.absolute_url(''))
            except Page.DoesNotExist:
                pass
        else:
            try:
                ta = TemplateAsset.objects.get(template=page.template, initial_path=search_path)
                html_content = html_content.replace(link, ta.file.url)
            except TemplateAsset.DoesNotExist:
                pass

    name = page.html.name.split('/')[-1]
    page.html.delete()
    page.html.save(name, ContentFile(html_content.encode('utf-8')))

    if css_assets:
        for asset in css_assets:
            css_content = asset.file.read().decode('utf-8')
            links = re.findall(CSS_LINK_PATTERN, css_content)
            for link in links:
                nl = link.replace('../', '').lstrip('url(').rstrip(')').strip('\'').strip('"').strip('.').strip('/')
                nl = nl.split('?')[0]
                try:
                    link_inside = TemplateAsset.objects.get(initial_path=nl, template=asset.template)
                    css_content = css_content.replace(link, f'url({link_inside.file.url})')
                except TemplateAsset.DoesNotExist:
                    print(f'Link {nl} inside css is not registered.')
            name = asset.file.name.split('/')[-1]
            asset.file.delete()
            asset.file.save(name, ContentFile(css_content.encode('utf-8')))


def extract_zip(input_zip):
    input_zip = ZipFile(input_zip)
    return {name: input_zip.read(name) for name in input_zip.namelist()}



from django.db.models import Manager

from .querysets import UserBridgeQuerySet


class UserBridgeManager(Manager):
    def get_queryset(self):
        return UserBridgeQuerySet(self.model, using=self._db).filter(active=True)

    def everything(self):
        return super(UserBridgeManager, self).get_queryset()
